
#include "ButtonEvent.hpp"
#include "ButtonEventDispatcher.hpp"
#include <iostream>


//3. Allocate a variable in .data memory
int global_var = 10; //initialized => .data memory

//4. Allocate a variable in .bss memory
int global_var_not_initialized; // not initialized => .bss (memory is zero initialized before program execution)

// Function defined in the .text section (code section)
void text_function() {
    std::cout << "Hello from the .text section!" << std::endl;
}


// Sample button event handler functions
void ButtonHandler1(const ButtonEvent& e) {
    std::cout << "ButtonHandler1 called with type: " << static_cast<int>(e.getTypeId()) << std::endl;
}

void ButtonHandler2(const ButtonEvent& e) {
    std::cout << "ButtonHandler2 called with type: " << static_cast<int>(e.getTypeId()) << std::endl;
}


int main() {

/*
    //1. Allocate a variable in .stack memory
    int x = 5;
    std::cout << "Value of x: " << x << std::endl;

    //2.  Allocate a pointer variable pointing into .heap memory
    int* ptr = new int(2);

    // Access and use the variable through the pointer
    std::cout << "Value of variable on the heap: " << *ptr << std::endl;
    std::cout << "Memory address of ptr on the heap: " << &ptr << std::endl;
    std::cout << "Memory address of variable on the heap: " << ptr << std::endl;

    // Deallocate the variable when no longer needed
    delete ptr;

     // Access and use the global variable in .data
    std::cout << "Value of global_var in the .data memory segment: " << global_var << std::endl;

    // Access and use the global variable in .bss
    std::cout << "Value of global_var_not_initialized: " << global_var_not_initialized << std::endl;

    //5. Allocate a pointer variable pointing into .text section
    // Pointer variable pointing to a function in the .text section
    void (*ptr_to_function)() = &text_function;

    // Call the function through the pointer
    ptr_to_function();
*/

    //******************************************************************************************
    //******************************************************************************************
    //******************************************************************************************

     std::cout << "Hello from Homework!" << std::endl;

     // Create ButtonEvent objects
    ButtonEvent shortPress(ButtonEvent::PressTypeId::SHORT_PRESS, ButtonEvent::ButtonSourceId::BUTTON_0);
    ButtonEvent longPress(ButtonEvent::PressTypeId::LONG_PRESS, ButtonEvent::ButtonSourceId::BUTTON_1);
    ButtonEvent veryLongPress(ButtonEvent::PressTypeId::VERY_LONG_PRESS, ButtonEvent::ButtonSourceId::BUTTON_0);

    // Access and print the type and source identifiers
    std::cout << "Button 1: " << static_cast<int>(shortPress.getTypeId()) << ", " << static_cast<int>(shortPress.getSourceId()) << std::endl;
    std::cout << "Button 2: " << static_cast<int>(longPress.getTypeId()) << ", " << static_cast<int>(longPress.getSourceId()) << std::endl;
    std::cout << "Button 3: " << static_cast<int>(veryLongPress.getTypeId()) << ", " << static_cast<int>(veryLongPress.getSourceId()) << std::endl;

    // Create ButtonEventDispatcher object
    ButtonEventDispatcher dispatcher;

    // Register button event handler functions
    dispatcher.RegisterButtonHandler(ButtonHandler1);
    dispatcher.RegisterButtonHandler(ButtonHandler2);

    // Create a sample button event
    ButtonEvent event(ButtonEvent::PressTypeId::VERY_LONG_PRESS, ButtonEvent::ButtonSourceId::BUTTON_1);

    // ButtonHandler1(event);
    // ButtonHandler2(event);

    // Dispatch the event
    std::cout << "Dispatching button event..." << std::endl;
    dispatcher.Dispatch(event);

    // Unregister one of the handlers
    dispatcher.UnregisterButtonHandler(ButtonHandler1);

    // Dispatch the event again
    std::cout << "Dispatching button event after unregistering one handler..." << std::endl;
    dispatcher.Dispatch(event);


    return 0;
}