#include "ButtonEvent.hpp"
#include "ButtonEventDispatcher.hpp"

void ButtonEventDispatcher::Dispatch(const ButtonEvent& e) const
{
    for(ButtonEventCb h : handlers)
    {
        if(h) h(e);
    }
}

// handlers will be registered only once, i.e. mutiple registrations of the same handler will be ignored.
// attempts to register more than <cMaxHandlers> different handlers will be silently dropped.
void ButtonEventDispatcher::RegisterButtonHandler(ButtonEventCb cb)
{
    bool inserted = false;
    for(ButtonEventCb &h : handlers)
    {
        if(h == nullptr && !inserted) {
            h = cb; // insert at first empty slot
            inserted = true;
        } else if(h == cb) {
            if(inserted) {
                h = nullptr; // invalidate later occurrence
            }
            break;
        }
    }
}

void ButtonEventDispatcher::UnregisterButtonHandler(ButtonEventCb cb)
{
    for(ButtonEventCb &h : handlers)
    {
        if(h == cb) {
            h = nullptr; // remove
            break;
        }
    }
}
