#include <iostream>
#include "Button.hpp"


void fct1(const ButtonEvent& input) {
  std::cout << "fct1 was called" << std::endl;
};
void fct2(const ButtonEvent& input) {
  std::cout << "fct2 was called" << std::endl;
};

void fct3(const ButtonEvent& input) {
  std::cout << "fct3 was called" << std::endl;
};
void fct4(const ButtonEvent& input) {
  std::cout << "fct4 was called" << std::endl;
};


ButtonEvent::ButtonEvent(typeIdType inputType, sourceIdType inputSource):
    typeId(inputType),
    sourceId(inputSource)
{
  std::cout << "ButtonEvent::ButtonEvent" << std::endl;
  std::cout << static_cast<char>(typeId) << static_cast<char>(sourceId) << std::endl;
};

ButtonEventDispatcher::ButtonEventDispatcher()
{
  std::cout << "ButtonEventDispatcher::ButtonEventDispatcher" << std::endl;
};


void ButtonEventDispatcher::RegisterButtonHandler(callbackType myFct)
{
  // TODO: add a safety to avoid allowing the register multiple time the same function?
  std::cout << "ButtonEventDispatcher::RegisterButtonHandler" << std::endl;

  for(auto& fct : handlerFunctions) {   // Range-for!
    if(fct == nullptr) {
      fct = reinterpret_cast<callbackType>(myFct);
      std::cout << "Added callback" << std::endl;
      return;
    }
  }
  std::cout << "Cannot register more handlers, silently droping new callback function" << std::endl;
};


void ButtonEventDispatcher::UnregisterButtonHandler(callbackType myFct)
{
  std::cout << "ButtonEventDispatcher::UnregisterButtonHandler" << std::endl;

  for(auto& fct : handlerFunctions) {   // Range-for!
    if(fct == myFct) {
      fct = nullptr;
      std::cout << "Unregistered callback" << std::endl;
      return;
    }
  }
  std::cout << "No valid callback to unregister, avoiding doing it" << std::endl;
};


void ButtonEventDispatcher::Dispatch(const ButtonEvent &e) const
{
  std::cout << "ButtonEventDispatcher::Dispatch" << std::endl;

  for(auto& fct : handlerFunctions) {   // Range-for!
    if(fct != nullptr) {
      fct(e);
      std::cout << "Called register callback ..." << std::endl;
    }
  }
};


int main()
{
  typeIdType myType = typeIdType::cShortPress;
  sourceIdType mySource = sourceIdType::cButtonZero;
  ButtonEvent myButton {
    myType,
    mySource
  };

  std::cout << "0" << std::endl;
  ButtonEventDispatcher myDispatcher = ButtonEventDispatcher();
  std::cout << "Register fct1:" << std::endl;
  myDispatcher.RegisterButtonHandler(fct1);
  std::cout << "Register fct2:" << std::endl;
  myDispatcher.RegisterButtonHandler(fct2);
  std::cout << "Register fct3:" << std::endl;
  myDispatcher.RegisterButtonHandler(fct3);
  std::cout << "Unregister fct1:" << std::endl;
  myDispatcher.UnregisterButtonHandler(fct1);
  std::cout << "Unregister fct2:" << std::endl;
  myDispatcher.UnregisterButtonHandler(fct2);
  std::cout << "Unregister fct4:" << std::endl;
  myDispatcher.UnregisterButtonHandler(fct4);
  std::cout << "Register fct1:" << std::endl;
  myDispatcher.RegisterButtonHandler(fct1);
  //myDispatcher.RegisterButtonHandler(intotint());
  std::cout << "Trigger dispatcher, expect fct1 and fct3:" << std::endl;
  myDispatcher.Dispatch(myButton);
  std::cout << "Program over" << std::endl;
  return 0;
}