#include <iostream>
#include <string>
#include <cassert>
#include "ButtonEventArgs.hpp"
#include "EventDispatcher.hpp"

std::string getTypeAsString(ButtonEventArgs::Type type)
{
    switch (type)
    {
        case ButtonEventArgs::Type::cShortPress: return "Short Press";
        case ButtonEventArgs::Type::cLongPress: return "Long Press";
        case ButtonEventArgs::Type::cVeryLongPress: return "Very Long Press";

        default: return "Unidentified Type";
    }
}

std::string getSourceAsString(ButtonEventArgs::Source source)
{
    switch (source)
    {
        case ButtonEventArgs::Source::cButton0: return "Button 0";
        case ButtonEventArgs::Source::cButton1: return "Button 1";

        default: return "Unidentified Source";
    }
}

void genericButtonHandler(int index, const ButtonEventArgs& e) {
    std::cout
    << "Received Button Event on Handler #"
    << index
    << " - Type: "
    << getTypeAsString(e.type)
    << ", Source: "
    << getSourceAsString(e.source)
    << "\n";
}

void buttonHandler0(const ButtonEventArgs& e) { genericButtonHandler(0, e); }
void buttonHandler1(const ButtonEventArgs& e) { genericButtonHandler(1, e); }
void buttonHandler2(const ButtonEventArgs& e) { genericButtonHandler(2, e); }
void buttonHandler3(const ButtonEventArgs& e) { genericButtonHandler(3, e); }
void buttonHandler4(const ButtonEventArgs& e) { genericButtonHandler(4, e); }

int main()
{
    EventDispatcher<ButtonEventArgs, 4> dispatcher;
    ButtonEventArgs shortPressEvent(ButtonEventArgs::Type::cShortPress, ButtonEventArgs::Source::cButton0);
    ButtonEventArgs longPressEvent(ButtonEventArgs::Type::cLongPress, ButtonEventArgs::Source::cButton1);
    ButtonEventArgs veryLongPressEvent(ButtonEventArgs::Type::cVeryLongPress, ButtonEventArgs::Source::cButton0);

    // Registering 4 handlers, all shall pass
    assert(dispatcher.RegisterCallback(buttonHandler0) == true);
    assert(dispatcher.RegisterCallback(buttonHandler1) == true);
    assert(dispatcher.RegisterCallback(buttonHandler2) == true);
    assert(dispatcher.RegisterCallback(buttonHandler3) == true);

    // Registering a new handler shall fail
    assert(dispatcher.RegisterCallback(buttonHandler4) == false);

    // Registering an already registered handler shall pass even though there are no available handler slots
    assert(dispatcher.RegisterCallback(buttonHandler0) == true);

    std::cout << "Should print 4 lines (0, 1, 2, 3):\n";
    dispatcher.Dispatch(shortPressEvent);
    std::cout << std::endl;

    // Unregistering 2 already registered handlers
    dispatcher.UnregisterCallback(buttonHandler1);
    dispatcher.UnregisterCallback(buttonHandler2);

    // Unregistering 1 not registered handler, shall silently do nothing
    dispatcher.UnregisterCallback(buttonHandler4);

    std::cout << "Should print 2 lines (0, 3):\n";
    dispatcher.Dispatch(longPressEvent);
    std::cout << std::endl;

    // Registering a new handler shall pass
    assert(dispatcher.RegisterCallback(buttonHandler4) == true);

    // Registering an already registered handler shall pass
    assert(dispatcher.RegisterCallback(buttonHandler3) == true);

    std::cout << "Should print 3 lines (0, 4, 3):\n";
    dispatcher.Dispatch(veryLongPressEvent);
    std::cout << std::endl;
}
